#include <graph.h>
#include <ctype.h>
#include <stdio.h>
#define TRUE 1
#define FALSE 0
#define COM_1 0x3F8

#define X_OFFSET 1
#define Y_OFFSET 129
#define X_VIEWPORT 70
#define Y_VIEWPORT 0
/* global variables */
char buffer[2000]; /* holds raw data from scope */

struct waveform
	{
	char valid;
	char inv;
	char var;
	char cplg[10];
	char volts_verbose[10];
	float volts_number;
	char readout;
	char xy;
	char x10mag;
	char seconds_verbose[10];
	float seconds_number;
	char trig_position[6];
	char slope;
	char trig_source[6];
	char trig_mode[10];
	char time_out;
	char chan_sel;
	char recalled;
	char valid_store;
	char acq_mode[6];
	char store_mode;
	char auto_trig;
	char wave_data[1500];
	char comment_long[127];
	char comment_title[20];
	}
	struct waveform chanel_1,chanel_2,ref_1,ref_2,ref_3,ref_4;
	struct waveform *wavebank;
	char full_grid=TRUE;

	struct cursors
	{
	int position;
	char data[2024];
	}
	struct cursors cursor_1,cursor_2;
	struct cursors *cursor;






/* prototypes */
void display_grid(void);
void print_screen(void);

void init_serial(void);
char rec_serial(void);
void send_serial(char);
void send_serial_string();
void rec_serial_string();
void talk_scope(void);
void fun_key_menu(int);
void decode_scope(struct waveform *);
void draw_curve(struct waveform *);
int hex_to_int(char *);
void update_graph(void);
void cursor_up (struct cursors *);
void cursor_down(struct cursors *);
void move_cursors(void);
void update_cursors(void);
void update_text(void);
void update_cursors_units(float);
void do_cursor(struct waveform *);
void read_from_disk(void);
main()
{
	char key;
	char funkey;
	/* init wave forms */
	chanel_1.valid=FALSE;
	chanel_2.valid=FALSE;
	ref_1.valid=FALSE;
	ref_2.valid=FALSE;
	ref_3.valid=FALSE;
	ref_4.valid=FALSE;

	cursor_1.position=10;
	cursor_2.position=490;
	_setvideomode(_ERESCOLOR);     /* switch into ega color */
	_settextrows(43);
	_setviewport(X_VIEWPORT,Y_VIEWPORT,X_VIEWPORT+512,Y_VIEWPORT+256);

	init_serial();
	full_grid=TRUE;
	display_grid();
	while (1)
	{
	/* get some input */
		fun_key_menu(1);
		key=getch();

		/* detirmine what function is selected */

		switch(key)
		{
			case 'q':
			case 'Q':

				_setvideomode(_TEXTC80);
				exit(0);
				break;
		}
		funkey=0;
		if ((key ==0 ) || (key==0xE0))
		{
			funkey=getch();
		}
		switch(funkey)
		{
			case 0:
			break;

			case 0x3b: /* F1 > Grid */
				full_grid=++full_grid & 1;
				update_graph();

				break;

			case 0x3c: /* F2 > Print  */
				print_screen();
				break;

			case 0x3d: /* F3          */
				break;

			case 0x3e: /* F4          */
				break;

			case 0x3f: /* F5          */
				break;

			case 0x40: /* F6 Scope    */
				talk_scope();
				break;

			case 0x41: /* F7          */
				move_cursors();
				break;

			case 0x42: /* F8          */
				break;

			case 0x43: /* F9          */
				break;

			case 0x44: /* F10 > EXIT  */
				_setvideomode(_DEFAULTMODE);
				exit(0);
				break;

		}
	}
}



/***********************/
/* large functions     */

void talk_scope()
/* talk to the scope (i.e. get wave form ) */
{
	 char funkey;

	 fun_key_menu(2);
	 while (getch()!=0);

	 funkey=getch();
	 switch(funkey)
	 {
		case 0x3b:
			strcpy(buffer,"CURV? CH1;");
			send_serial_string();
			rec_serial_string();
			wavebank=&chanel_1;
			decode_scope(wavebank);
			update_graph();
			update_text();
		break;

		case 0x3c:
			strcpy(buffer,"CURV? CH2;");
			send_serial_string();
			rec_serial_string();
			wavebank=&chanel_2;
			decode_scope(wavebank);
			update_graph();
			update_text();
		break;

		case 0x3d:
			strcpy(buffer,"CURV? REF1;");
			send_serial_string();
			rec_serial_string();
			/*read_from_disk();*/
			wavebank=&ref_1;
			decode_scope(wavebank);
			update_graph();
			update_text();
		break;

		case 0x3e:
			strcpy(buffer,"CURV? REF2;");
			send_serial_string();
			rec_serial_string();
			wavebank=&ref_2;
			decode_scope(wavebank);
			update_graph();
			update_text();
		break;

		case 0x3f:
			strcpy(buffer,"CURV? REF3;");
			send_serial_string();
			rec_serial_string();
			wavebank=&ref_3;
			decode_scope(wavebank);
			update_graph();
			update_text();
		break;

		case 0x40:
			strcpy(buffer,"CURV? REF4;");
			send_serial_string();
			rec_serial_string();
			wavebank=&ref_4;
			decode_scope(wavebank);
			update_graph();
			update_text();
		break;






	 }

}


void move_cursors()
{
	int inuse;
	fun_key_menu(3);
	while (1)
	{
		switch( getch())
		{
			case 0x3b:
				inuse=1;
				break;
			case 0x3c:
				inuse=2;
				break;
			case 0x3d:
				inuse=3;
				break;
			case 0x3e:
				if ((inuse & 1) && (cursor_1.position >0))
				{
					cursor=&cursor_1;
					cursor_down(cursor);
					cursor_1.position--;
					cursor_up(cursor);
				}
				if (( inuse & 2) && (cursor_2.position > cursor_1.position+1))
				{
					cursor=&cursor_2;
					cursor_down(cursor);
					cursor_2.position--;
					cursor_up(cursor);
				}
				break;
			case 0x3f:
				if ((inuse & 1) && (cursor_1.position+1 < cursor_2.position))
				{
					cursor=&cursor_1;
					cursor_down(cursor);
					cursor_1.position++;
					cursor_up(cursor);
				}
				if (( inuse & 2 ) && (cursor_2.position <500) )
				{
					cursor=&cursor_2;
					cursor_down(cursor);
					cursor_2.position++;
					cursor_up(cursor);
				}
				break;
			case 0x40:
				if ((inuse & 1) && (cursor_1.position >10))
				{
					cursor=&cursor_1;
					cursor_down(cursor);
					cursor_1.position-=10;
					cursor_up(cursor);
				}
				if (( inuse & 2) && (cursor_2.position > cursor_1.position+10))
				{
					cursor=&cursor_2;
					cursor_down(cursor);
					cursor_2.position-=10;
					cursor_up(cursor);
				}
				break;
			case 0x41:
				if ((inuse & 1) && (cursor_1.position+10 < cursor_2.position))
				{
					cursor=&cursor_1;
					cursor_down(cursor);
					cursor_1.position+=10;
					cursor_up(cursor);
				}
				if (( inuse & 2 ) && (cursor_2.position <490) )
				{
					cursor=&cursor_2;
					cursor_down(cursor);
					cursor_2.position+=10;
					cursor_up(cursor);
				}
				break;

			case 0x44:
				return;
				break;
		}
		update_cursors();
	}
}











void decode_scope(struct waveform *destin)
/* take the data in buffer and break down it to the scope struck*/
{
	char temp;
	int pointer;
	int tempint;
	/* find begining of the data */
	pointer=1;
	while(buffer[pointer++] != ':');

	/*if channel 2 then bump up pointer */
	if(destin == &chanel_2)
		pointer+=3;

	/* get inverted status */
	destin->inv=(buffer[pointer] & 0x08);

	/* get calibration status */
	destin->var=(buffer[pointer] & 0x04);

	/* get coupling status */
	temp=(buffer[pointer++] & 0x03);
	switch(temp)
	{
		case 0:
			strcpy(destin->cplg," DC");
			break;
		case 1:
			strcpy(destin->cplg," AC");
			break;
		case 2:
			strcpy(destin->cplg,"GND");
			break;
		case 3:
			strcpy(destin->cplg,"OFF");
			break;
	}

	/* find volts/div */
	temp=(buffer[pointer++] & 0x0f);
	switch(temp)
	{
		case 0:
			destin->volts_number=0.005;
			strcpy(destin->volts_verbose,"  5 mV");
			break;
		case 1:
			destin->volts_number=0.010;
			strcpy(destin->volts_verbose," 10 mV");
			break;
		case 2:
			destin->volts_number=0.020;
			strcpy(destin->volts_verbose," 20 mV");
			break;
		case 3:
			destin->volts_number=0.050;
			strcpy(destin->volts_verbose," 50 mV");
			break;
		case 4:
			destin->volts_number=0.100;
			strcpy(destin->volts_verbose," 0.1 V");
			break;
		case 5:
			destin->volts_number=0.200;
			strcpy(destin->volts_verbose," 0.2 V");
			break;
		case 6:
			destin->volts_number=0.500;
			strcpy(destin->volts_verbose," 0.5 V");
			break;
		case 7:
			destin->volts_number=1.000;
			strcpy(destin->volts_verbose," 1.0 V");
			break;
		case 8:
			destin->volts_number=2.000;
			strcpy(destin->volts_verbose," 2.0 V");
			break;
		case 9:
			destin->volts_number=5.000;
			strcpy(destin->volts_verbose," 5.0 V");
			break;
		case 10:
			destin->volts_number=10.00;
			strcpy(destin->volts_verbose," 10  V");
			break;
		case 11:
			destin->volts_number=20.00;
			strcpy(destin->volts_verbose," 20  V");
			break;
		case 12:
			destin->volts_number=50.00;
			strcpy(destin->volts_verbose," 50  V");
			break;
	}
	/* if not channel_2 then bump pointer */
	if (destin != &chanel_2)
		   pointer+=3;

	/* get the reaadout */
	temp=buffer[pointer];
	destin->readout=(temp &0x08);

	/*get the xy dealie */
	destin->xy=(temp & 0x04);

	/* get x10 mag. */
	destin->x10mag=(temp &02);

	/* get time base */
	temp =(temp & 0x01);
	temp=temp*16;
	temp+=(buffer[pointer++] & 0x0f);
	switch (temp)
	{
		case 0:
			strcpy(destin->seconds_verbose,"50  nS");
			destin->seconds_number=0.000000050;
			break;
		case 1:
			strcpy(destin->seconds_verbose,"0.1 uS");
			destin->seconds_number=0.000000100;
			break;
		case 2:
			strcpy(destin->seconds_verbose,"0.2 uS");
			destin->seconds_number=0.000000200;
			break;
		case 3:
			strcpy(destin->seconds_verbose,"0.5 uS");
			destin->seconds_number=0.000000500;
			break;
		case 4:
			strcpy(destin->seconds_verbose,"1.0 uS");
			destin->seconds_number=0.000001000;
			break;
		case 5:
			strcpy(destin->seconds_verbose,"2.0 uS");
			destin->seconds_number=0.000002000;
			break;
		case 6:
			strcpy(destin->seconds_verbose,"5.0 uS");
			destin->seconds_number=0.000005000;
			break;
		case 7:
			strcpy(destin->seconds_verbose,"10  uS");
			destin->seconds_number=0.000010000;
			break;
		case 8:
			strcpy(destin->seconds_verbose,"20  uS");
			destin->seconds_number=0.000020000;
			break;
		case 9:
			strcpy(destin->seconds_verbose,"50  uS");
			destin->seconds_number=0.000050000;
			break;
		case 10:
			strcpy(destin->seconds_verbose,"0.1 mS");
			destin->seconds_number=0.000100000;
			break;
		case 11:
			strcpy(destin->seconds_verbose,"0.2 mS");
			destin->seconds_number=0.000200000;
			break;
		case 12:
			strcpy(destin->seconds_verbose,"0.5 mS");
			destin->seconds_number=0.000500000;
			break;
		case 13:
			strcpy(destin->seconds_verbose,"1.0 mS");
			destin->seconds_number=0.001000000;
			break;
		case 14:
			strcpy(destin->seconds_verbose,"2.0 mS");
			destin->seconds_number=0.002000000;
			break;
		case 15:
			strcpy(destin->seconds_verbose,"5.0 mS");
			destin->seconds_number=0.005000000;
			break;
		case 16:
			strcpy(destin->seconds_verbose,"10  mS");
			destin->seconds_number=0.010000000;
			break;
		case 17:
			strcpy(destin->seconds_verbose,"20  mS");
			destin->seconds_number=0.020000000;
			break;
		case 18:
			strcpy(destin->seconds_verbose,"50  mS");
			destin->seconds_number=0.050000000;
			break;
		case 19:
			strcpy(destin->seconds_verbose,"0.1  S");
			destin->seconds_number=0.100000000;
			break;
		case 20:
			strcpy(destin->seconds_verbose,"0.2  S");
			destin->seconds_number=0.200000000;
			break;
		case 21:
			strcpy(destin->seconds_verbose,"0.5  S");
			destin->seconds_number=0.500000000;
			break;
		case 22:
			strcpy(destin->seconds_verbose,"1.0  S");
			destin->seconds_number=1.000000000;
			break;
		case 23:
			strcpy(destin->seconds_verbose,"2.0  S");
			destin->seconds_number=2.000000000;
			break;
		case 24:
			strcpy(destin->seconds_verbose,"5.0  S");
			destin->seconds_number=5.000000000;
			break;
		case 25:
			strcpy(destin->seconds_verbose,"10   S");
			destin->seconds_number=10.00000000;
			break;
		case 26:
			strcpy(destin->seconds_verbose,"20   S");
			destin->seconds_number=20.00000000;
			break;
	}

	/* get the trigger position */
	temp=(buffer[pointer]&0x0c);
	temp = temp/4;
	switch(temp)
	{
		case 0:
			strcpy(destin->trig_position,"POST");
			break;
		case 1:
			strcpy(destin->trig_position,"MID ");
			break;
		case 2:
			strcpy(destin->trig_position,"PRE ");
			break;
	}
	/* get slope */
	destin->slope=(buffer[pointer] &0x02);

	/* get trigger source */
	temp=(buffer[pointer++] & 0x01);
	temp =temp *2;
	temp =temp +((buffer[pointer] &0x08) /8);
	switch(temp)
	{
		case 0:
			strcpy(destin->trig_source,"VERT");
			break;
		case 1:
			strcpy(destin->trig_source,"CH 1");
			break;
		case 2:
			strcpy(destin->trig_source,"CH 2");
			break;
		case 3:
			strcpy(destin->trig_source,"EXT ");
			break;
	}

	/* get trigger mode */
	temp=(buffer[pointer++] & 0x07);
	switch(temp)
	{
		case 0:
			strcpy(destin->trig_mode,"NORM  ");
			break;
		case 1:
			strcpy(destin->trig_mode,"AUTO LVL");
			break;
		case 2:
			strcpy(destin->trig_mode,"AUTO BL");
			break;
		case 3:
			strcpy(destin->trig_mode,"SSEQ  ");
			break;
	}

	/* GET TIME OUT */
	temp=buffer[pointer++];
	destin->time_out=(temp & 0x08);

	/* get channel select */
	destin->chan_sel=(temp & 0x04);

	/*get recalled wave form */
	destin->recalled=(temp & 0x02);

	/* get valid store */
	destin->valid_store=(temp &0x01);

	/* get aquisition mode */
	temp=(buffer[pointer]& 0x0c);
	temp=temp/4;
	switch(temp)
	{
		case 0:
			strcpy(destin->acq_mode,"NORM");
			break;
		case 1:
			strcpy(destin->acq_mode,"ENV ");
			break;
		case 2:
			strcpy(destin->acq_mode,"AVG ");
			break;
		case 3:
			strcpy(destin->acq_mode,"C ENV");
			break;
	}

	/* get store mode */
	temp=(buffer[pointer] & 0x02);
	destin->store_mode=temp;

	/*get auto trig */
	temp=(buffer[pointer++] & 0x01);
	destin->auto_trig=temp;
	/* all modes should be broken out now */

	tempint=0;
	pointer+=6;
	while ((destin->wave_data[tempint++]=buffer[pointer++]) != ';');
	destin->valid=1;
}


void draw_curve(struct waveform *destin)
/* plot a wave form */
{
	char hex[2];
	int y;
	int pointer=0;
	hex[0]=destin->wave_data[pointer++];
	hex[1]=destin->wave_data[pointer++];
	y= hex_to_int(hex)-128;
	y*=-1.25;
	_moveto_w(0+X_OFFSET,y+Y_OFFSET);
	while (pointer<1024)
	{
		hex[0]=destin->wave_data[pointer++];
		hex[1]=destin->wave_data[pointer++];
		y= hex_to_int(hex)-128;
		y*=-1.25;
		_lineto_w(pointer/2+X_OFFSET,y+Y_OFFSET);
	}
}

void update_graph()

/* erase and redraw the screen */
{
	_clearscreen(_GVIEWPORT);
	display_grid();

/* CHECK IF CHANEL 1 IS VALID AND IS DISPLAYED */
	if (chanel_1.valid == TRUE )
	{
		wavebank=&chanel_1;
		_setcolor(3);
		draw_curve(wavebank);
	}
	if (chanel_2.valid==TRUE)
	{
		wavebank=&chanel_2;
		_setcolor(4);
		draw_curve(wavebank);
	}
	if (ref_1.valid==TRUE)
	{
		wavebank=&ref_1;
		_setcolor(5);
		draw_curve(wavebank);
	}
	if (ref_2.valid==TRUE)
	{
		wavebank=&ref_2;
		_setcolor(6);
		draw_curve(wavebank);
	}
	if (ref_3.valid==TRUE)
	{
		wavebank=&ref_3;
		_setcolor(9);
		draw_curve(wavebank);
	}
	if (ref_4.valid==TRUE)
	{
		wavebank=&ref_4;
		_setcolor(10);
		draw_curve(wavebank);
	}
}

void update_text()
{
	_settextposition(34,0);
	if (chanel_1.valid)
	{
		_setcolor(12);
		printf("Ch 1 %6s/Div %6s/div \n",chanel_1.seconds_verbose,chanel_1.volts_verbose);
	}
	if (chanel_2.valid)
	{
		_setcolor(13);
		printf("CH 2 %6s/Div %6s.div \n",chanel_2.seconds_verbose,chanel_2.volts_verbose);
	}
	if (ref_1.valid)
	{
		printf("RF 1 %6s/Div %6s.div \n",ref_1.seconds_verbose,ref_1.volts_verbose);
	}
	if (ref_2.valid)
	{
		printf("RF 2 %6s/Div %6s/div \n",ref_2.seconds_verbose,ref_2.volts_verbose);
	}
	if (ref_3.valid)
	{
		printf("RF 3 %6s/Div %6s/div \n",ref_3.seconds_verbose,ref_3.volts_verbose);
	}
	if (ref_4.valid)
	{
		printf("RF 4 %6s/Div %6s/div \n",ref_4.seconds_verbose,ref_4.volts_verbose);
	}
}


void cursor_up( struct cursors *destin)
{
	 int temp;


	_getimage(destin->position+6,0,destin->position+6,255,destin->data);

	_setcolor(15);
	_moveto(destin->position+6,0);
	_lineto(destin->position+6,255);
}

void cursor_down( struct cursors *destin)
{
	_putimage(destin->position+6,0,destin->data,_GPSET);
}

void update_cursors()
{
	int temp;
	_settextposition(33,33);
	printf("time    volts     volts     volts");
	temp=34;
	if (chanel_1.valid)

	{
		_settextposition(temp++,30);
		wavebank=&chanel_1;
		do_cursor(wavebank);
	}
	if (chanel_2.valid)
	{
		_settextposition(temp++,30);
		wavebank=&chanel_2;
		do_cursor(wavebank);
	}
	if (ref_1.valid)
	{
		_settextposition(temp++,30);
		wavebank=&ref_1;
		do_cursor(wavebank);
	}
	if (ref_2.valid)
	{
		_settextposition(temp++,30);
		wavebank=&ref_2;
		do_cursor(wavebank);
	}
	if (ref_3.valid)
	{
		_settextposition(temp++,30);
		wavebank=&ref_3;
		do_cursor(wavebank);
	}
	if (ref_4.valid)
	{
		_settextposition(temp++,30);
		wavebank=&ref_4;
		do_cursor(wavebank);
	}
}



void do_cursor(struct waveform *destin)
{
	float time_1;
	float time_2;
	float volt_1;
	float volt_2;
	char volts[2];

	time_1=((float)cursor_1.position/50)* destin->seconds_number;
	time_2=((float)cursor_2.position/50)* destin->seconds_number;
	update_cursors_units(time_2-time_1);
	printf("S  ");
	volts[0]=destin->wave_data[cursor_1.position*2 +6];
	volts[1]=destin->wave_data[cursor_1.position*2 +7];
	volt_1=(float)hex_to_int(volts);
	volt_1=(float)destin->volts_number*(volt_1-128)/30;
	update_cursors_units(volt_1);
	printf("V  ");
	volts[0]=destin->wave_data[cursor_2.position*2 +6];
	volts[1]=destin->wave_data[cursor_2.position*2 +7];
	volt_2=(float)hex_to_int(volts);
	volt_2=(float)destin->volts_number*(volt_2-128)/30;
	update_cursors_units(volt_2);
	printf("V  ");
	update_cursors_units(volt_2-volt_1);
	printf("V  ");

}

void update_cursors_units(float number)
{
	char unit;

	char minus;
	unit=' ';
	minus=FALSE;
	if(number <0.0)
	{
		number=number*-1;
		minus=TRUE;
	}
	if (number < (.1))
	{
		number *=1000;
		unit='m';
	}
	if (number <.10)
	{
		number *=1000;
		unit='u';
	}
	if (number <.10)
	{
		number *=1000;
		unit='n';
	}
	if (number <.10)
	{
		number*=1000;
		unit='p';
	}
	if (minus)
	{
		number=number*-1;
	}
	printf("%6.2f%c",number,unit);
}


/****************************/
/* varoius small functions  */

void display_grid()
{

	int  row;
	unsigned  collum;
	unsigned  tics;
	unsigned  step=10;
	short x,y;

	_setcolor(7);
	if (full_grid)
	{
		step=1;
	}
	for(row=0;row<11;row+=step)
	{
		x=(short)row*50+6+X_OFFSET;
		y=(short)0+Y_OFFSET-120;
		_moveto_w(x,y);
		_lineto_w((row*50+6+X_OFFSET),(240+Y_OFFSET-120));
	}
	step=8;
	if (full_grid)
	{
		step=1;
	}
	for (collum=0;collum<9;collum+=step)
	{
		_moveto_w(6+X_OFFSET,(collum*30)+Y_OFFSET-120);
		_lineto_w(500+6+X_OFFSET,collum*30+Y_OFFSET-120);
	}
	for(tics=1;tics<50;tics++)
	{
		_moveto_w(tics*10+6+X_OFFSET,2+Y_OFFSET);
		_lineto_w(tics*10+6+X_OFFSET,Y_OFFSET-2);
	}
	for(tics=1;tics<40;tics++)
	{
		_moveto_w(5*50-2+6+X_OFFSET,tics*(30/5)+Y_OFFSET-120);
		_lineto_w(5*50+2+6+X_OFFSET,(tics*(30/5))+Y_OFFSET-120);
	}
	for(tics=1;tics<50;tics++)
	{
		_setpixel_w(tics*50/5+6+X_OFFSET,Y_OFFSET-(.8*90));
		_setpixel_w(tics*50/5+6+X_OFFSET,Y_OFFSET+(.8*90));
	}
}




void print_screen(void)
 {
	register int col;
	register int row;
	register char byte;
	/* set the view port to whole screen */
	_setviewport(0,0,639,349);
	for (row=0;row<371;row+=8)
	{       fputs("\x1bL\x80\x02",stdprn);
		for (col=0;col <640;col++)
		{
			byte=0;
			if (_getpixel(col,row) !=0)
				{byte=byte+128;};
			if (_getpixel(col,(row+1)) !=0)
				{byte=byte+64;};
			if (_getpixel(col,(row+2)) !=0)
				{byte=byte+32;};
			if (_getpixel(col,(row+3)) !=0)
				{byte=byte+16;};
			if (_getpixel(col,(row+4)) !=0)
				{byte=byte+8;};
			if (_getpixel(col,(row+5)) !=0)
				{byte=byte+4;};
			if (_getpixel(col,(row+6)) !=0)
				{byte=byte+2;};
			if (_getpixel(col,(row+7)) !=0)
				{byte=byte+1;};
			fputc(byte,stdprn);
		};
		fputs("\x1b\x33\x18\n",stdprn);
		};
		fputs("\x0c",stdprn);
		/* put view port back the way we got it */
		_setviewport(X_VIEWPORT,Y_VIEWPORT,X_VIEWPORT+512,Y_VIEWPORT+256);

}

/* serial drivers */

void init_serial(void)
/* set up com1 for 9600 */
{
	outp(COM_1+3,0x80);
	outp(COM_1+0,12);
	outp(COM_1+1.0);
	outp(COM_1+3,0x03);
      /*  COM_1 set to 9600 none 1 */
}


char rec_serial()
/* wait for one char from the serial port */
{
	char data=0;
	while ((data & 0x01) !=1)
	{
		data=inp(COM_1+5);
	}
	data=inp(COM_1);
	return(data);
}
void send_serial(char data)
/* wiat for trams empty then send data out com1 */
{

	while ((inp(COM_1+5) & 0x40) != 0x40);
	outp(COM_1,data);
}

void send_serial_string()
/* transmit a \0 terminated string out COM_1 */
{
	int pointer=0;
	char byte;
	while (buffer[pointer]!=0)
	{
		byte=buffer[pointer];
		send_serial(byte);
		pointer++;
	}
}

void rec_serial_string()
/*wait for a 0x0c terminated string */
{
	int pointer=0;
	char data=0;
	while(data != ';')
	{
		data=rec_serial();
		buffer[pointer++]=data;
	}
	buffer[pointer]=0x00;
}

void read_from_disk()
/* pull in a waveform from disk*/

{
	FILE *infile;
	int pointer=0;
	char data=0;
	char inname[15];

	printf("Wave form file name :");
	gets(inname);
	infile=fopen(inname,"r");
	while (1)
	{
		if feof(infile)
			break;
		data=getc(infile);
		buffer[pointer++]=data;
	}
	buffer[pointer]=0x00;
}
















void fun_key_menu(int menu)
/* display menu # menu */

{

	_settextposition(42,1);
		printf(" F1       F2      F3      F4      F5     F6       F7      F8      F9      F10");
	_settextposition(43,1);
	switch (menu)
	{

	    /*            F1      F2      F3      F4      F5      F6      F7      F8      F9      F10 */
	    /* formater 1234567 1234567 1234567 1234567 1234567 1234567 1234567 1234567 1234567 1234567 1234567 1234567 */
		case 1:
		printf(" Grid    Print   Load    Save   Comment  Scope  Cursors  Erase            EXIT ");
		break;
		case 2:
		printf(" CH 1      CH 2    REF1    REF2    REF3    REF4                          RETURN");
		break;
		case 3:
		printf(" Left    Right   Both     <-      ->    <<-      ->>                     RETURN");

	    }

}


/* convert from hex string to int */
int hex_to_int(char *hex)
{
	int temp;
	char nib;
	nib=hex[1];
	if (nib > '9')
		nib=nib-7;

	temp=(int)(nib & 0x0f);

	nib=hex[0];
	if (nib> '9')
		nib-=7;
	temp+=(int)((nib & 0x0f ) * 16);
	return (temp);
}

